/*
	xVideo DX8 effects test
	Copyright (c) 2010-2011 Cristea Aurel Ionut
*/

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <math.h>
#include "xVideo.h"

HWND win=NULL;

DWORD chan;			// channel handle
HFX fx[4];			// 3 eq bands + reverb

OPENFILENAME ofn;

// display error messages
void Error(const char *es)
{
	char mes[200];
	sprintf(mes,"%s\n(error code: %d)",es,xVideo_ErrorGetCode());
	MessageBox(win,mes,0,0);
}


#define MESS(id,m,w,l) SendDlgItemMessage(win,id,m,(WPARAM)(w),(LPARAM)(l))

void UpdateFX(int b)
{
	int v=MESS(20+b,TBM_GETPOS,0,0);
	if (b<3) {
		xVideoFX_PARAMEQ p;
		xVideo_GetFXParameters(chan,fx[b],&p);
		p.fGain=10.0-v;
		xVideo_SetFXParameters(chan,fx[b],&p);
	} else {
		xVideoFX_REVERB p;
		xVideo_GetFXParameters(chan,fx[3],&p);
		p.fReverbMix=(v<20?log(1-v/20.0)*20:-96);
		xVideo_SetFXParameters(chan,fx[3],&p);
	}
}

BOOL CALLBACK dialogproc(HWND h,UINT m,WPARAM w,LPARAM l)
{
	switch (m) {
		case WM_COMMAND:
			switch (LOWORD(w)) {
				case IDCANCEL:
					DestroyWindow(h);
					break;
				case 10:
					{
						char file[MAX_PATH]="";
						ofn.lpstrFilter="0All files\0*.*\0\0";
						ofn.lpstrFile=file;
						if (GetOpenFileName(&ofn)) {
							// free both MOD and stream, it must be one of them! :)
			
							xVideo_StreamFree(chan);

							if (!(chan=xVideo_StreamCreateFile(file,0,0,0))) {					
								// whatever it is, it ain't playable
								MESS(10,WM_SETTEXT,0,"click here to open a file...");
								Error("Can't play the file");
								break;
							}
							MESS(10,WM_SETTEXT,0,file);
							{ // setup the effects
								xVideoFX_PARAMEQ p;
								fx[0]=xVideo_ChannelSetFX(chan,xVideo_DX8_PARAMEQ);
								fx[1]=xVideo_ChannelSetFX(chan,xVideo_DX8_PARAMEQ);
								fx[2]=xVideo_ChannelSetFX(chan,xVideo_DX8_PARAMEQ);
								fx[3]=xVideo_ChannelSetFX(chan,xVideo_DX8_REVERB);
								p.fGain=0;
								p.fBandwidth=18;
								p.fCenter=125;
								xVideo_SetFXParameters(chan,fx[0],&p);
								p.fCenter=1000;
								xVideo_SetFXParameters(chan,fx[1],&p);
								p.fCenter=8000;
								xVideo_SetFXParameters(chan,fx[2],&p);
								UpdateFX(0);
								UpdateFX(1);
								UpdateFX(2);
								UpdateFX(3);
							}
							xVideo_ChannelPlay(chan);
						}
					}
					break;
			}
			break;

		case WM_VSCROLL:
			if (l) {
				UpdateFX(GetDlgCtrlID((HWND)l)-20);
			}
			break;

		case WM_INITDIALOG:
			win=h;
			memset(&ofn,0,sizeof(ofn));
			ofn.lStructSize=sizeof(ofn);
			ofn.hwndOwner=h;
			ofn.nMaxFile=MAX_PATH;
			ofn.Flags=OFN_HIDEREADONLY|OFN_EXPLORER;
			// setup output - default device
			if (!xVideo_Init(win,0)) {
				Error("Can't initialize xVideo");
				DestroyWindow(win);
				break;
			}
			
			// initialize eq/reverb sliders
			MESS(20,TBM_SETRANGE,FALSE,MAKELONG(0,20));
			MESS(20,TBM_SETPOS,TRUE,10);
			MESS(21,TBM_SETRANGE,FALSE,MAKELONG(0,20));
			MESS(21,TBM_SETPOS,TRUE,10);
			MESS(22,TBM_SETRANGE,FALSE,MAKELONG(0,20));
			MESS(22,TBM_SETPOS,TRUE,10);
			MESS(23,TBM_SETRANGE,FALSE,MAKELONG(0,20));
			MESS(23,TBM_SETPOS,TRUE,20);
			return 1;

		case WM_DESTROY:
			xVideo_Free();
			break;
	}
	return 0;
}

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
	 

	{ // enable trackbar support
		INITCOMMONCONTROLSEX cc={sizeof(cc),ICC_BAR_CLASSES};
		InitCommonControlsEx(&cc);
	}
    //Set the NULL video renderer :)

	xVideo_SetConfig(xVideo_CONFIG_VideoRenderer,xVideo_NULLVideo);
	DialogBox(hInstance,(char*)1000,0,&dialogproc);

	return 0;
}
