/*
	xVideo console WAV writer
	Copyright (c) 2010-2011 Cristea Aurel Ionut
*/
#include "Windows.h"
#include <stdlib.h>
#include <stdio.h>
#include "xVideo.h"
 
#include <conio.h>
 

#ifdef _BIG_ENDIAN
inline DWORD le_32(DWORD v)
{
	return (v>>24)|((v>>8)&0xff00)|((v&0xff00)<<8)|(v<<24);
}
inline WORD le_16(WORD v)
{
	return (v>>8)|(v<<8);
}
#else
#define le_32(v) (v)
#define le_16(v) (v)
#endif

// display error messages
void Error(const char *text) 
{
	printf("Error(%d): %s\n",xVideo_ErrorGetCode(),text);
	xVideo_Free();
	exit(0);
}

void main(int argc, char **argv)
{
	xVideo_ChannelInfo info;
	DWORD chan,p;
	QWORD pos;
	FILE *fp;
	short buf[10000];
	WAVEFORMATEX wf;

	printf("xVideo WAV writer example\n"
			"-------------------------------------------------\n");

	 

	if (argc!=2) {
		printf("\tusage: writewav <file>\n");
		return;
	}

 
	// setup output - "no sound" device, 44100hz, stereo, 16 bits
	if (!xVideo_Init(0,0))
		Error("Can't initialize xVideo");
    //set NULL video renderer ... much faster when decoding audio :D

	xVideo_SetConfig(xVideo_CONFIG_VideoRenderer,xVideo_NULLVideo);
	// try streaming the file/url
	if ((chan=xVideo_StreamCreateFile(argv[1],0,0,xVideo_STREAM_DECODE))) {
		pos=xVideo_ChannelGetLength(chan,xVideo_POS_MILISEC);
		if (pos==-1) // length not available
			printf("streaming file");
		else

			printf("streaming file [%I64u miliseconds]",pos);
	}

	// display the time length
	if (pos && pos!=-1) {
		p=pos/1000.0f;
		printf(" %u:%02u\n",p/60,p%60);
	} else // no time length available
		printf("\n");

	if (!(fp=fopen("out.WAV","wb"))) Error("Can't create file");
	printf("writing to out.WAV file... press a key to stop\n");
	// write WAV header
	xVideo_ChannelGetInfo(chan,&info);
	wf.wFormatTag=1;
	wf.nChannels=info.nChannels;
	wf.wBitsPerSample=info.wBits;
	wf.nBlockAlign=wf.nChannels*wf.wBitsPerSample/8;
	wf.nSamplesPerSec=info.freq;
	wf.nAvgBytesPerSec=wf.nSamplesPerSec*wf.nBlockAlign;
#ifdef _BIG_ENDIAN // swap byte order
	wf.wFormatTag=le_16(wf.wFormatTag);
	wf.nChannels=le_16(wf.nChannels);
	wf.wBitsPerSample=le_16(wf.wBitsPerSample);
	wf.nBlockAlign=le_16(wf.nBlockAlign);
	wf.nSamplesPerSec=le_32(wf.nSamplesPerSec);
	wf.nAvgBytesPerSec=le_32(wf.nAvgBytesPerSec);
#endif
	fwrite("RIFF\0\0\0\0WAVEfmt \20\0\0\0",20,1,fp);
	fwrite(&wf,16,1,fp);
	fwrite("data\0\0\0\0",8,1,fp);
    xVideo_ChannelPlay(chan);
	while (!_kbhit() && (xVideo_ChannelGetState(chan) == xVideo_STATE_PLAYING)) {
		int c=xVideo_ChannelGetData(chan,(BYTE*)buf,20000);
#ifdef _BIG_ENDIAN
		if (!(info.flags&xVideo_SAMPLE_8BITS)) // swap 16-bit byte order
			for (p=0;p<c/2;p++) buf[p]=le_16(buf[p]);
#endif
		fwrite(buf,1,c,fp);
		pos=xVideo_ChannelGetPosition(chan,xVideo_POS_MILISEC);
#ifdef _WIN32
		printf("pos %09I64u\r",pos);
#else
		printf("pos %09llu\r",pos);
#endif
		fflush(stdout);
	}
	// complete WAV header
	fflush(fp);
	p=ftell(fp);
	fseek(fp,4,SEEK_SET);
	putw(le_32(p-8),fp);
	fflush(fp);
	fseek(fp,40,SEEK_SET);
	putw(le_32(p-44),fp);
	fflush(fp);
	fclose(fp);

	xVideo_Free();
}
