﻿using System;
using System.Text;
using System.Runtime.InteropServices;
 

namespace xVideo
{

    /// Callbacks
    /// 
    /// 
    public delegate void CallBackEnumerator([param: MarshalAs(UnmanagedType.LPStr)] string name, Guid guid, IntPtr user);
    /*
      name - the device name
      GUID - the device CLSID
      user - The 'user' parameter
    */
    public delegate void CallBackEnumProfiles([param: MarshalAs(UnmanagedType.LPStr)] string profile,IntPtr user);
    /*
     profile - the device profile
     user - The 'user' parameter 
    */
    public delegate void CallBackConnectedFilters(IntPtr Filter, [param: MarshalAs(UnmanagedType.LPStr)] string filtername,
                   int PP, IntPtr user);
    /*
     Filter- the instance of a IBaseFilter;
     FilterName - The name of the Filter
     pp - Boolean, Filter supports or not Property Pages
     user - The 'user' parameter 
    */
   public delegate void  DSPPROC(int Dhandle,uint channel,IntPtr buffer,int lenght,IntPtr user);
    /* DSP callback function. NOTE: A DSP function should obviously be as quick as
       possible... other DSP functions, streams can not be processed
       until it's finished.
       handle : The DSP handle
       channel: Channel that the DSP is being applied to
       buffer : Buffer to apply the DSP to
       length : Number of bytes in the buffer
       user   : The 'user' parameter given when calling xVideo_ChannelSetDSP
    */
    public delegate void DVPPROC(uint handle, uint channel, IntPtr buffer, uint length,uint dataType,int width,int height,IntPtr user);
    /* DVP callback function. NOTE: A DVP function should obviously be as quick as
       possible... other DVP functions, streams can not be processed
       until it's finished.
       handle     : The DVP handle
       channel    : Channel that the DVP is being applied to
       buffer     : Buffer to apply the DVP to
       length     : Number of bytes in the buffer
       dataType   : one of the xVideo_TYPE_xxx
       width      : video width
       height     : video height
       user       : The 'user' parameter given when calling xVideo_ChannelSetDVP
    */
    public delegate int CallBackStreamEnumerator(uint format, [param: MarshalAs(UnmanagedType.LPStr)] string  name,uint index,int enabled,IntPtr user);
    /*
       format - 0: unknown, 1: video , 2: audio , 3: subtitle
       name -  the format name
       index - stream index. Use this index to enable or disable it
       enabled -is this stream enabled??
       user - The 'user' parameter
    */
    public delegate void CallBackSync(uint chan,uint sync,uint data,IntPtr user);
    /*
     chan - the channel that triggered the sync
     sync - one of the xVideo_SYNC_XXX
     data - sync data
     user - the 'user' parameter 
    */

    public struct xVideo_StreamFlags
    {
     public const uint xVideo_UNICODE		  =	0x80000000; //Use this parameter at flags when create a stream
     public const uint xVideo_STREAM_AUTOFREE = 0x40000;    //Use this parameter at flags when create a stream
     public const uint xVideo_STREAM_DECODE   = 0x200000;   //Use this parameter at flags when create a stream
     public const uint xVideo_STREAM_MIX      = 0x1000000;  //Use this flag to enable mixing video on a channel
     public const uint xVideo_NOAUDIO_PROC    = 0x80000;    //Use this flag to disable audio processor
     public const uint xVideo_STREAM_AUTODVD  = 0x4000000;  //use this flag to enable auto dvd functions(on mouse down, keys etc)
     public const uint xVideo_STREAM_LOOP     = 0x8000000;  //use this flag to restart stream when it's finished
     public const uint xVideo_STREAM_VIDEOPROC = 0x20000;    //Use this to enable video processor
    }
//xVideo_CaptureXXX functions flags
//...........................................
    public struct xVideo_CaptureFlags
    {
      public const uint xVideo_CaptureAudio      =    0x10066;
      public const uint xVideo_CaptureVideo      =    0x10080;
    }

//xVideo_ChannelGetState return values
    public struct xVideo_StateModes
    {
      public const uint xVideo_STATE_PLAYING	= 1; //channel is playing
      public const uint xVideo_STATE_PAUSED  	= 2; //channel is paused
      public const uint xVideo_STATE_STOPPED    = 3; //channel is stopped
    }

//xVideo_ChannelGetLenght/SetPosition/GetPosition: modes]
    public struct xVideo_Position_Modes
    {
      public const uint xVideo_POS_SEC     = 0;  //position in seconds
      public const uint xVideo_POS_FRAME   = 1; //position in frames
      public const uint xVideo_POS_MILISEC = 2; //position in miliseconds
      public const uint xVideo_POS_REFTIME = 3; //position in reference time
    }
//xVideo_SetConfig configs and values\\ 
//...........................................
    public struct xVideo_Configs
    {
        public const uint xVideo_CONFIG_VideoRenderer = 0x1000;
        public const uint xVideo_NULLVideo            = 6;   //pass this to select NULL Video Renderer
        public const uint xVideo_EVR                  = 5;   //pass this to select Enhanced Video Renderer
        public const uint xVideo_VMR9WindowsLess      = 4;   //pass this to select VMR9 window less render
        public const uint xVideo_VMR7WindowsLess      = 3;   //pass this to select VMR7 window less render
        public const uint xVideo_VMR9                 = 2;  //pass this to select VMR9 Windowed render
        public const uint xVideo_VMR7                 = 1;  //pass this to select VMR7 Windowed render
 //............................................
        public const uint xVideo_CONFIG_AudioRenderer = 0x1004;  //use this to select audio renderer
        public const uint xVideo_NULLAudio            = 0x1432;  //use this to select NULL Audio Renderer
        public const uint xVideo_DefaultAudio         = 0x1433;  //use this to select Windows Default Audio Device
//.............................................
        public const uint xVideo_CONFIG_FLOATDSP      = 0x1005;
 
//.............................................
        public const uint xVideo_CONFIG_WindowLessStreams   =  0x1002;  //use this to set the number of streams in a VMR7/9 Windowsless mode
        public const uint xVideo_CONFIG_WindowLessHandle    =  0x1001;  //VMR7/VMR9 WindowLess Mode need an initial window so set a HWND to use properly VMR
    }



    //////Color Controls FLAGS////////////////////////////
    public struct xVideo_ColorFlags
    {
       public const uint xVideo_ControlBrightness  = 0x00000001;    //used to set Audio Volume
       public const uint xVideo_ControlContrast   = 0x00000002;    //used to set Audio Pan
       public const uint xVideo_ControlHue        = 0x00000004;    //used to set Graph Rate
       public const uint xVideo_ControlSaturation = 0x00000008;   //use this to set main video alpha blend value
    }

    ///////xVideo_ChannelSetAttribute/GetAttribute Constants
    public struct xVideo_ChannelAttributes
    {
       public const uint xVideo_ATTRIB_VOL   = 1;    //used to set Audio Volume
       public const uint xVideo_ATTRIB_PAN   = 2;    //used to set Audio Pan
       public const uint xVideo_ATTRIB_RATE  = 3;    //used to set Graph Rate
       public const uint xVideo_ATTRIB_ALPHA = 4;   //use this to set main video alpha blend value
    }

    public struct xVideo_GetDataType
    {
        public const uint xVideo_DATA_END = 0x80000000;	// end of file
        public const uint xVideo_DATA_FLOAT  =   0x40000000;	// flag: return floating-point sample data
        public const uint xVideo_DATA_FFT256  =   0x60000001;   // flag: return FFT 256
        public const uint xVideo_DATA_FFT512   =  0x60000002;  // flag: return FFT 512 
        public const uint xVideo_DATA_FFT1024  =  0x60000003;   // flag: return FFT 1024 
    }
   
    //xVideo_CallbackItemByIndex
    public struct xVideo_CALLBACKINDEX_TYPES
    {
       public const int xVideo_CALLBACK_AC     = 1;  //used to get the audio capture device
       public const int xVideo_CALLBACK_VC     = 2;  //used to get the Video capture device
       public const int xVideo_CALLBACK_AR     = 3;  //used to get the audio renderer device
    }


    //FX effect types, use with xVideo_ChannelSetFX
    public struct xVideoFX_TYPES
    {
       public const int xVideo_FX_REVERB     = 1;
       public const int xVideo_FX_PARAMEQ    = 2;
       public const int xVideo_FX_CHORUS     = 3;
       public const int xVideo_FX_COMPRESSOR = 4;
       public const int xVideo_FX_ECHO       = 5;
       public const int xVideo_FX_FLANGER    = 6;
       public const int xVideo_FX_PITCHTEMPO = 7;
       public const int xVideo_FX_CHANMIX    = 8;
    }

 
    public struct xVideoFX_PARAMEQ
    {
        public float   fCenter;
        public float   fBandwidth;
        public float   fGain;
    } 
    public struct xVideoFX_TEMPO
    {
        public float tempo;
        public float pitch;
        public float tempoRate;
    } 
    public struct xVideoFX_CHANNELMIX
    {
        public uint mode;  //xVideo_CHANMIX_xxx
    } 
   
    public struct xVideoFX_REVERB
    {
        public float   fInGain;                // [-96.0,0.0]            default: 0.0 dB
        public float   fReverbMix;             // [-96.0,0.0]            default: 0.0 db
        public float   fReverbTime;            // [0.001,3000.0]         default: 1000.0 ms
        public float   fHighFreqRTRatio;       // [0.001,0.999]          default: 0.001
    } 
   
 
    public struct xVideoFX_FLANGER
    {
       public float       fWetDryMix;
       public float       fDepth;
       public float       fFeedback;
       public float       fFrequency;
       public long        lWaveform;
       public float       fDelay;
       public long       lPhase;
    } 
  
    public struct xVideoFX_CHORUS
    {
        public float fWetDryMix;
        public float fDepth;
        public float fFeedback;
        public float fFrequency;
        public long  lWaveform;          // 0=triangle, 1=sine
        public float fDelay;
        public long  lPhase;   //xVideo_PHASE_xxx
    } 
   
 
    public struct xVideoFX_COMPRESSOR
    {
        public float fGain;
        public float fAttack;
        public float fRelease;
        public float fThreshold;
        public float fRatio;
        public float fPredelay;
    } 

    public struct xVideoFX_ECHO
    {
        public float fWetDryMix;
        public float fFeedback;
        public float fLeftDelay;
        public float fRightDelay;
        public long lPanDelay;
    } 

    public struct xVideoFX_PHASE_Modes
    {
       public const int xVideo_PHASE_NEG_180       = 0;
       public const int xVideo_PHASE_NEG_90        = 1;
       public const int xVideo_PHASE_ZERO          = 2;
       public const int xVideo_PHASE_90            = 3;
       public const int xVideo_PHASE_180           = 4;
    }
 
    //xVideoFX_CHANNELMIX modes 
    public struct xVideoFX_CHANNELMIX_Modes
    {
       public const int xVideo_CHANMIX_STEREO      = 0; //unchanged 
       public const int xVideo_CHANMIX_LEFTRIGHT   = 1; //copy left audio data to right
       public const int xVideo_CHANMIX_RIGHTLEFT   = 2; //copy right audio data to left
    }
    public struct xVideo_ChannelInfo
    {
        public float AvgTimePerFrame; //average Time per frame (i.e 23.97 - it will draw 23.97 frames per seconds);
        public int  Height, Width; //video size;
        public int  nChannels; //number of channels (i.e.1= mono,2= stereo...)
        public uint freq;  //sample rate
        public uint wBits; // number of bits per sample of mono data */
        public bool floatingpoint;//true if audio is 32 bit floating point, false else
    }

    //for xVideo_ChannelColorRange function////
    public struct xVideo_ColorsRange{
		public float  MinValue;
        public float  MaxValue;
        public float  DefaultValue;
        public float  StepSize;
		public uint   type; // one of the xVideo_ControlXXX
    }

    //for xVideo_ChannelSetColors function////
    public struct xVideo_ColorsSet
    {
        public float Contrast;
        public float Brightness;
        public float Hue;
        public float Saturation;
    }

    //for Video BMP functions////
    public struct xVideo_VideoBitmap
    {
        public int visible;
        public int inLeft;
        public int inTop;
        public int inRight;
        public int inBottom;
        public float outLeft;
        public float outTop;
        public float outRight;
        public float outBottom;
        public float alphavalue;
        public IntPtr transColor;
        public IntPtr bmp;
    }
    //xVideo_PLUGININFO
    public struct xVideo_PLUGININFO
    {
        public uint version;				  // version 
        public uint decoderType;			  // decoder type: 1=audio 2=video
        public string  plgdescription;          // plugin description
    }
    public struct xVideo_STREAMS
    {
        public uint format;  //0: unknown, 1: video , 2: audio , 3: subtitle
        public string name;   // the format name
        public uint index;   //stream index. Use this index to enable or disable it
        public int enabled;  //is this stream enabled??
    }
 
 
    //for xVideo_DVDChannelMenu function
    public struct xVideo_DVDMenuFlags
    {
        public const int xVideo_DVDSelectAtPos  = 21;
        public const int xVideo_DVDActionAtPos  = 22;
        public const int xVideo_DVDActivateBut  = 23;
        public const int xVideo_DVDSelectButton = 24;
    }

   
    //for xVideo_DVDSetProperty function
    public struct xVideo_DVDSetPropertyFlags
    {
       public const int xVideo_DVD_TITLEMENU       = 100;
       public const int xVideo_DVD_ROOT            = 101;      //go to DVD root
       public const int xVideo_DVD_NEXTCHAPTER     = 102;      //go to dvd next chapter
       public const int xVideo_DVD_PREVCHAPTER     = 103;    //go to dvd previous chapter
       public const int xVideo_DVD_TITLE           = 104;     //go to dvd title
       public const int xVideo_DVD_TITLECHAPTER    = 105;     //play chapter in current title
    }

    //for xVideo_DVDGetProperty function 
    public struct xVideo_DVDGetPropertyFlags
    {
       public const uint xVideo_CurentDVDTitle          =   0x10010;
       public const uint xVideo_DVDTitles               =   0x10020;
       public const int xVideo_DVDTitleChapters         =   0x10030;
       public const int xVideo_DVDCurrentTitleDuration  =   145;
       public const int xVideo_DVDCurrentTitlePosition  =   146;
    }

    public struct xVideo_Errors
    {
 
        public const int xVideo_OK               =  0; 
        public const int xVideo_INVALIDCHAN      =  1; 
        public const int xVideo_UNKNOWN          =  2;
        public const int xVideo_NotInitialized   =  3;
        public const int xVideo_POSNOTAVAILABLE  =  4;
        public const int xVideo_NODSHOW          =  5; 
        public const int xVideo_INVALIDWINDOW    =  6;
        public const int xVideo_NOAUDIO          =  7;
        public const int xVideo_NOVIDEO          =  8;
        public const int xVideo_ERRORMEM         =  9;
        public const int xVideo_ERRORCALLBACK    =  10;
        public const int xVideo_ERRORFLAG        =  11;
        public const int xVideo_NOTAVAILABLE     =  12;
        public const int xVideo_ERRORINIT        =  13;
    }



    /////xVideo_TYPE_XXX//////////////////////////////////
    public struct xVideo_TYPE
    {
     public const int xVideo_TYPE_RGB24       =  1;
     public const int xVideo_TYPE_RGB32        =  2;
     public const int xVideo_TYPE_YUYV        =  3;
     public const int xVideo_TYPE_IYUV        =  4;
     public const int xVideo_TYPE_YVU9        =  5;
     public const int xVideo_TYPE_YV12        =  6;
     public const int xVideo_TYPE_NV12        =  7;
     public const int xVideo_TYPE_UYVY        =  8;
    }


/////xVideo_SYNC_XXX//////////////////////////////////
    public struct xVideo_SYNC
    {
       public const int xVideo_SYNC_END                          =  1;
       public const int xVideo_SYNC_DVD_CHAPTER_START            =  2;
       public const int xVideo_SYNC_DVD_TITLE_CHANGE             =  3;
       public const int xVideo_SYNC_DVD_ERROR                    =  4;
       public const int xVideo_SYNC_REPAINT                      =  5;
       public const int xVideo_SYNC_VIDEO_SIZE_CHANGED           =  6;
    }

    public class Functions
    {
        const string dllname = "xVideo.dll";

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ErrorGetCode();

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_GetVersion();

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_Init(IntPtr handle,uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_Free();

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_StreamCreateFile([param: MarshalAs(UnmanagedType.LPWStr)] string str, uint pos,IntPtr winHandle, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelPlay(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelPause(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelStop(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_StreamFree(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelAddWindow(uint chan, IntPtr win);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelRemoveWindow(uint chan, uint win);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static double xVideo_ChannelGetLength(uint chan, uint mode);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelSetPosition(uint chan, double pos, uint mode);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelSetPositionVB(uint chan, Int64 pos, uint mode);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static double xVideo_ChannelGetPosition(uint chan, uint mode);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr xVideo_GetGraph(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelResizeWindow(uint chan, uint hWindow, int left, int top, int right, int bottom);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelSetFullscreen(uint chan, int value);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_SetConfig(uint option, uint value);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_GetConfig(uint option);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelGetInfo(uint chan, [param: In, Out] ref xVideo_ChannelInfo data);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static IntPtr xVideo_ChannelGetBitmap(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelColorRange(uint chan,uint id ,[param: In, Out] ref xVideo_ColorsRange data);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelSetColors(uint chan,uint id ,[param: In, Out] ref xVideo_ColorsSet data);
      
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_SetVideoAlpha(uint chan,uint  win,int layer,float alpha);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static float xVideo_GetVideoAlpha(uint chan, uint win, int layer);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_MIXChannelResize(uint chan, int layer, int left, int top, int right, int bottom);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_CaptureGetDevices(uint devicetype, CallBackEnumerator callback, IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_CaptureDeviceProfiles(int device, uint devicetype, CallBackEnumProfiles callback, IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelOverlayBMP(uint chan ,[param: In, Out] ref xVideo_VideoBitmap data);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelSetWindow(uint chan, uint hWindow, IntPtr handle);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelAddFile(uint chan,[param: MarshalAs(UnmanagedType.LPWStr)] string str, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelGetConnectedFilters(uint chan, CallBackConnectedFilters CALLBack, IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ShowFilterPropertyPage(uint chan, uint filter, IntPtr parent);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelGetState(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelSetFX(uint chan,int fx);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelRemoveFX(uint chan,uint fx);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_SetFXParameters(uint chan,uint fx,[param: In, Out] ref IntPtr data);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_GetFXParameters(uint chan, uint fx, [param: In, Out]ref IntPtr data);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static void xVideo_ChannelSetAttribute(uint chan, uint option, float value);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static float xVideo_ChannelGetAttribute(uint chan, uint option);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_CaptureCreate(int audio,int video,int audioprofile,int videoprofile, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_CaptureFree(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_GetAudioRenderers(CallBackEnumerator CALLBack, IntPtr user);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_StreamCreateFileMem(IntPtr data,Int64 size,IntPtr win,uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_StreamCreateFileUser(uint flags,IntPtr win,IntPtr proc,IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelRemoveDSP(uint chan,uint dsp);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelSetDSP(uint chan,DSPPROC proc,IntPtr user);
    
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelGetData(uint chan,IntPtr data,uint size);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelRepaint(uint chan, IntPtr handle, IntPtr hDC);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static string xVideo_CallbackItemByIndex(uint type,uint index);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_LoadPlugin([param: MarshalAs(UnmanagedType.LPWStr)] string filename, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_PluginGetInfo(uint plugin,[param: In, Out] ref xVideo_PLUGININFO info);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_RemovePlugin(uint plugin);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_StreamCreateDVD([param: MarshalAs(UnmanagedType.LPWStr)] string file,IntPtr win, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_DVDGetProperty(uint chan, uint prop, uint value);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_DVDSetProperty(int chan, uint option, uint value);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_DVDChannelMenu(uint chan, uint option, int value1, int value2);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelGetLevel(uint chan);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_StreamCreateFilter([param: MarshalAs(UnmanagedType.LPWStr)] string str,[param: MarshalAs(UnmanagedType.LPWStr)] string filter,IntPtr winHandle, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_LoadPluginDS([param: MarshalAs(UnmanagedType.LPWStr)] string guid, [param: MarshalAs(UnmanagedType.LPWStr)] string name,uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_VCamStreamChannel(uint chan,int stream);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_VCamPush(IntPtr win,IntPtr data);
 
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_Register([param: MarshalAs(UnmanagedType.LPWStr)] string email, [param: MarshalAs(UnmanagedType.LPWStr)] string reg, uint flags);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelCallBackSync(uint chan, CallBackSync callback, IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelSetDVP(uint chan, DVPPROC callback, IntPtr user);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelRemoveDVP(uint chan, uint dvp);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static uint xVideo_ChannelStreamsCount(uint chan);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelGetStream(uint chan, uint index, [param: In, Out] ref xVideo_STREAMS info);

        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelEnableStream(uint chan,uint index);
        
        [DllImport(dllname, CharSet = CharSet.Ansi, CallingConvention = CallingConvention.StdCall)]
        public extern static int xVideo_ChannelEnumerateStreams(uint chan,CallBackStreamEnumerator callback,IntPtr user);
 
    }

    public class Common
    {
        private uint chan; //instance channel
        private float mPan = 0;
        private float mRate = 1.0f;
        private float volume = 100; //volume; default: 100
        private float vAlpha = 100; //video alfa
        private string[] filters = new string[20];
        private int filtercount = -1;
        private int streamscount = -1;
        private xVideo_STREAMS []streams = new xVideo_STREAMS[20]; 
        public void SetWindow(IntPtr Win)
        {
            Functions.xVideo_ChannelSetWindow(chan, 0, Win);
        }
        private void CallBackConnectedFilters(IntPtr Filter, string filtername,
           int PP, IntPtr user)
        {
            filtercount++;
            filters[filtercount] = filtername;
        }
        int StreamEnumerator(uint format, string name, uint index, int enabled, IntPtr user)
        {
            streamscount++;
            streams[streamscount].name = name;
            streams[streamscount].enabled = enabled;
            streams[streamscount].index = index;
            streams[streamscount].format = format;
            return 1;
        }
        public void SetPan(int pan)
        {
            mPan = pan;
            Functions.xVideo_ChannelSetAttribute(chan, xVideo_ChannelAttributes.xVideo_ATTRIB_PAN, pan);
        }
        public void SetRate(float rate)
        {
            mRate = rate;
            Functions.xVideo_ChannelSetAttribute(chan,xVideo_ChannelAttributes.xVideo_ATTRIB_RATE, rate);
        }

        public void SetVolume(float vol)
        {
            volume = vol;
            Functions.xVideo_ChannelSetAttribute(chan, xVideo_ChannelAttributes.xVideo_ATTRIB_VOL, vol);
        }
        public void SetVideoAlpha(float alpha)
        {
            vAlpha = alpha;
            Functions.xVideo_ChannelSetAttribute(chan, xVideo_ChannelAttributes.xVideo_ATTRIB_ALPHA, alpha);
        }
        public void SetChannel(uint channel)
        {
            chan = channel;
            SetVolume(100);
            SetRate(1.0f);
            SetPan(0);
            filtercount = -1;
            Functions.xVideo_ChannelGetConnectedFilters(chan, CallBackConnectedFilters, (IntPtr)null);
            Functions.xVideo_ChannelEnumerateStreams(chan, StreamEnumerator, (IntPtr)null);
        }
        public bool Play()
        {
            return Functions.xVideo_ChannelPlay(chan) > 0;
        }
        public bool Pause()
        {
            return Functions.xVideo_ChannelPause(chan) > 0;
        }
        public bool Stop()
        {
            return Functions.xVideo_ChannelStop(chan) > 0;
        }
        public double Duration(uint mode)
        {
            return Functions.xVideo_ChannelGetLength(chan, mode);
        }
        public void SetPosition(double pos, uint mode)
        {
            Functions.xVideo_ChannelSetPosition(chan, pos, 0);
        }
        public double GetPosition(uint mode)
        {
            return Functions.xVideo_ChannelGetPosition(chan, mode);
        }
        public uint AddWindow(IntPtr win)
        {
            return Functions.xVideo_ChannelAddWindow(chan, win);
        }
        public void ResizeWindow(uint window,int left, int top, int right, int bottom)
        {
            Functions.xVideo_ChannelResizeWindow(chan,window, left, top, right, bottom);
        }
        public uint GetState()
        {
            return Functions.xVideo_ChannelGetState(chan);
        }
        public xVideo_ChannelInfo GetInfo(int ninfo)
        {
            xVideo_ChannelInfo info = new xVideo_ChannelInfo();
            Functions.xVideo_ChannelGetInfo(chan,ref info);
            return info;
        }
        public int GetFilterCount()
        {
            return filtercount;
        }
        public string GetFilterName(int filter)
        {
            if (filtercount < 0) return "";
            return filters[filter];
        }
        public void ShowProperyPage(int filter, IntPtr win)
        {
            Functions.xVideo_ShowFilterPropertyPage(chan, (uint)filter, win);
        }

        public bool StreamFree()
        {
            return Functions.xVideo_StreamFree(chan) > 0;
        }
        public uint GetChannel()
        {
            return chan;
        }

        public uint GetStreamsCount()
        {
            return (uint)streamscount;
        }

        public xVideo_STREAMS GetStreamInfo(int index)
        {
            return streams[index];
        }

    }
}
